module AdvancedDCL
  module Contour
    def param_init
      @param = Parameter.new(Common.param)
      @param.init("indexmj", Fixnum, "indexmj", "indexmjget", 3, 2)
      @param.init("indexmn", Fixnum, "indexmn", "indexmnget", 3, 2)
      @param.init("label", TrueClass, "udlset", "udlget", 0, 0)
      @param.init("lcent", TrueClass, "udlset", "udlget", 0, 0)
      @param.init("ldash", TrueClass, "udlset", "udlget", 0, 0)
      @param.init("idash", Fixnum, "udiset", "udiget", 0, 0)
      @param.init("isolid", Fixnum, "udiset", "udiget", 0, 0)
      @param.init("icycle", Fixnum, "udiset", "udiget", 0, 0)
      @param.init("nlev", Fixnum, "udiset", "udiget", 0, 0)
      @param.init("labelsize", Float, "labelsize", "labelsizeget", 3, 2)
      @param.init("lmsg", TrueClass, "udlset", "udlget", 0, 0)
      @param.init("titlesize", Float, "titlesize", "titlesizeget", 3, 2)
      @param.init("xttl", Float, "udrset", "udrget", 0, 0)
      @param.init("iundef", Fixnum, "udiset", "udiget", 0, 0)
      @param.init("labelfmt", String, "udsfmt", "udqfmt", 1, 1)

      @temp_param = @param.dup
      @backup = @param.dup
    end
    
    private :param_init

#------------------- method for parameters----------------------

    def set(key, val)
      @param[key] = val
    end

    def get(key)
      @param[key]
    end

    def gcla(max, min, dx)
      @param.sync

      DCL.udgcla(max, min, dx)
    end

    def gclb(z, dx)
      @param.sync

      DCL.udgclb(z, z.shape[0], z.shape[0], z.shape[1], dx)
    end

    def sclv(zlev, index, itype, clv, hl)
      @param.sync

      DCL.udsclv(zlev, index, itype, clv, hl)
    end

    def dclv(zlev)
      DCL.uddclv(zlev)
    end

    def iclv
      DCL.udiclv
    end

    def intervalget(nlev)
      DCL.rudlev(nlev)
    end
    
    def levelget
      nelv = DCL.udqcln
      levels = []
      for i in 0..nlev-1
	status = DCL.udqclv(i)
	levels.push(status[0])
      end
      levels
    end

#----------------------------------------------------------
#------------ draw method ----------------------
    def draw(data, x=nil, y=nil, temp=nil)
      if(!(y.is_a?(Hash)))
	if(x.is_a?(Hash))
	  temp = x.dup
	  x = nil
	elsif(Basics.typearray(x))
	  if(!Basics.typearray(y))
	    raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	  end
	end
      else
	raise ArgumentError, "both x & y must be Array or NArray(or nil)"
      end

      nx = data.shape[0]
      ny = data.shape[1]

      if(Basics.typearray(x) && Basics.typearray(y))
	DCL.uwsgxa(x, x.length)
	DCL.uwsgya(y, y.length)
	Basics.checkundef(x, y)
      else
	Basics.checkundef(nx.to_f, ny.to_f)
      end

      @param.sync
      @backup = @temp_param.set(temp, @param, @backup)
      @temp_param.sync

      Fig.settrf
    
      nb = DCL.glpget("nbitspw")
      nbr = (3*((nx+2)*(ny+2)*2.0/nb + 1)).ceil
      DCL.udcntz(data, nx, nx, ny, nbr)
      @backup.sync
      @temp_param.clear
    end

    module_function :param_init
    module_function :set
    module_function :get
    module_function :draw
    module_function :gcla
    module_function :gclb
    module_function :sclv
    module_function :dclv
    module_function :iclv
    module_function :intervalget
    module_function :levelget

    param_init

#------------------- singleton methods ----------------------
  
    def @param.indexmj(index)
      DCL.udiset("indxmj", index)
    end

    def @temp_param.indexmj(index)
      DCL.udiset("indxmj", index)
    end
    
    def @backup.indexmj(index)
      DCL.udiset("indxmj", index)
    end

    def @param.indexmjget
      DCL.udiget("indxmj")
    end

    def @param.indexmn(index)
      DCL.udiset("indxmn", index)
    end

    def @temp_param.indexmn(index)
      DCL.udiset("indxmn", index)
    end

    def @backup.indexmn(index)
      DCL.udiset("indxmn", index)
    end

    def @param.indexmnget
      DCL.udiget("indxmn")
    end

    def @param.labelsize(size)
      DCL.udrset("rsizel", size)
    end

    def @temp_param.labelsize(size)
      DCL.udrset("rsizel", size)
    end

    def @backup.labelsize(size)
      DCL.udrset("rsizel", size)
    end

    def @param.labelsizeget
      DCL.udrget("rsizel")
    end

    def @param.titlesize(size)
      DCL.udrset("rsizet", size)
    end

    def @temp_param.titlesize(size)
      DCL.udrset("rsizet", size)
    end

    def @backup.titlesize(size)
      DCL.udrset("rsizet", size)
    end

    def @param.titlesizeget
      DCL.udrget("rsizet")
    end

#----------------------------------------------------------

    @param.keys.each{|i|
      eval "def #{i}; @param['#{i}']; end"
      eval "def #{i}=(val); set('#{i}', val); end"
    }
  
    @param.keys.each{|i|
      eval "module_function :#{i}"
      eval "module_function :#{i}="
    }
  end
end
