module AdvancedDCL
  module Tone
    def param_init
      @param = Parameter.new(Common.param)

#--------------------- uepack parameter -------------------------
      @param.init("icolor1", Fixnum, "ueiset", "ueiget", 0, 0)
      @param.init("icolor2", Fixnum, "ueiset", "ueiget", 0, 0)
      @param.init("itpat", Fixnum, "ueiset", "ueiget", 0, 0)
      @param.init("nlev", Fixnum, "ueiset", "ueiget", 0, 0)
      @param.init("lbound", TrueClass, "uelset", "uelget", 0, 0)
      @param.init("ltone", TrueClass, "uelset", "uelget", 0, 0)
      @param.init("rlev", Float, "uerset", "uerget", 0, 0)

#--------------------- sgpack parameter -------------------------
      @param.init("lsoftf", TrueClass, "sglset", "sglget", 0, 0)
      @param.init("lcl2tn", TrueClass, "sglset", "sglget", 0, 0)
      @param.init("tnblen", Float, "sgrset", "sgrget", 0, 0)
      @param.init("irmode", Integer, "sgiset", "sgiget", 0, 0)

#--------------------- common parameter -------------------------
      @param.init("tonepat", Fixnum, "tonepat", "tonepatget", 3, 3)

      @temp_param = @param.dup
      @backup = @param.dup
      
    end
    
    private :param_init

#------------------- method for parameters----------------------
    
    def set(key, val)
      @param[key] = val
    end

    def get(key)
      @param[key] = val
    end

    def gtla(max, min, dx)
      @param.sync

      DCL.uegtla(max, min, dx)
    end

    def gtlb(z, dx)
      @param.sync

      DCL.uegtlb(z, z.shape[0], z.shape[0], z.shape[1], dx)
    end

    def stlv(tlev1, tlev2, ipat)
      @param.sync

      DCL.uestlv(tlev1, tlev2, ipat)
    end

    def stln(tlevn, ipatn)
      @param.sync

      if(!(tlevn.length == ipatn.length + 1))
	raise ArgumentError, "length mismatch between tone level & tone number"
      end
      DCL.uestln(tlevn, ipatn, ipath.length)
    end

    def itlv
      DCL.ueitlv
    end

#----------------------------------------------------------
#------------ draw method ----------------------

    def draw(data, x=nil, y=nil, temp=nil)
      if(!(y.is_a?(Hash)))
	if(x.is_a?(Hash))
	  temp = x.dup
	  x = nil
	elsif(Basics.typearray(x))
	  if(!Basics.typearray(y))
	    raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	  end
	end
      else
	raise ArgumentError, "both x & y must be Array or NArray(or nil)"
      end

      nx = data.shape[0]
      ny = data.shape[1]

      if(Basics.typearray(x) && Basics.typearray(y))
	DCL.uwsgxa(x, x.length)
	DCL.uwsgya(y, y.length)
	Basics.checkundef(x, y)
      else
	Basics.checkundef(nx.to_f, ny.to_f)
      end

      @param.sync
      @backup = @temp_param.set(temp, @param, @backup)
      @temp_param.sync

      Fig.settrf
    
      DCL.uetone(data, nx, nx, ny)

      @backup.sync
      @temp_param.clear
    end
    
    def imdraw(data, x=nil, y=nil, temp=nil)
      if(!(y.is_a?(Hash)))
	if(x.is_a?(Hash))
	  temp = x.dup
	  x = nil
	elsif(Basics.typearray(x))
	  if(!Basics.typearray(y))
	    raise ArgumentError, "both x & y must be Array or NArray(or nil)"
	  end
	end
      else
	raise ArgumentError, "both x & y must be Array or NArray(or nil)"
      end

      nx = data.shape[0]
      ny = data.shape[1]

      if(Basics.typearray(x) && Basics.typearray(y))
	DCL.uwsgxa(x, x.length)
	DCL.uwsgya(y, y.length)
	Basics.checkundef(x, y)
      else
	Basics.checkundef(nx.to_f, ny.to_f)
      end

      @param.sync
      @backup = @temp_param.set(temp, @param, @backup)
      @temp_param.sync

      Fig.settrf
    
      DCL.uetonf(data, nx, nx, ny)
      @backup.sync
      @temp_param.clear
    end

    def fill(upx, upy, temp=nil)
      Basics.checkundef(upx, upy)
      @param.sync
      @backup = @temp_param.set(temp, @param, @backup)
      @temp_param.sync

      Fig.settrf

      DCL.sgtnu(upx.length, upx, upy)
      @backup.sync
      @temp_param.clear
    end

    def vfill(vpx, vpy, temp=nil)
      Basics.checkundef(vpx, vpy)
      @param.sync
      @backup = @temp_param.set(temp, @param, @backup)
      @temp_param.sync

      DCL.sgtnv(vpx.length, vpx, vpy)
      @backup.sync
      @temp_param.clear
    end

    module_function :param_init
    module_function :set
    module_function :get
    module_function :draw
    module_function :imdraw
    module_function :gtla
    module_function :gtlb
    module_function :stlv
    module_function :itlv
    module_function :fill
    module_function :vfill

    param_init

#----------------------------------------------------------
    
    def @param.tonepat(itpat)
      DCL.ueiset("itpat", itpat)
      DCL.sgstnp(itpat)
    end

    def @temp_param.tonepat(itpat)
      DCL.ueiset("itpat", itpat)
      DCL.sgstnp(itpat)
    end

    def @backup.tonepat(itpat)
      DCL.ueiset("itpat", itpat)
      DCL.sgstnp(itpat)
    end

    def @param.tonepatget
      DCL.ueiget("itpat")
    end

    @param.keys.each{|i|
      eval "def #{i}; @param['#{i}']; end"
      eval "def #{i}=(val); set('#{i}', val); end"
    }
  
    @param.keys.each{|i|
      eval "module_function :#{i}"
      eval "module_function :#{i}="
    }
  end
end
